/************************************************
* FACTOR - Lookup table demonstration using the	*
* factorial series.								*
*												*
* 900228 MCMason - originally written			*
************************************************/

#include <stdio.h>
#include <stdlib.h>	/* exit						*/
#include "timer.h"	/* t1Start, t1Get, t1Format	*/

#define TRUE		(1==1)
#define BIG_FACT	1753	/* largest #!		*/
#define szDBL		sizeof(double)

long double table[1754];

/************************************************
* factorial - compute the factorial of a number	*
*												*
* INP:	i - the # to compute the factorial of	*
* OUT:	The factorial of the number, unless a	*
*		number greater than BIG_FACT or less	*
*		than zero is passed in (which results	*
*		in 0.0).								*
************************************************/
long double factorial(int i)
	{
	int			k;
	long double	f;

	if (i==0)
		return 1.0;
	else if (i>0 && i<BIG_FACT)
		for (f=1.0,k=1; k<=i; k++)
			f *= (long double)k;
	else return 0.0;
	return f;
	}

/************************************************
* main - generate & save the table of factorials*
* (if GENERATE_TABLE is defined), then load the	*
* table, and time how long it takes:			*
*	(a) to compute EACH factorial (0!-169!) 50	*
*		times, and 								*
*	(b) to look up each factorial 10000 times.	*
************************************************/
void main(void)
	{
	int  i, j, n;
	long double	f;
	long t1, t2, t3;
	FILE *of;

	/* STEP 1 - compute each factorial 5 times	*/
	puts("Now computing each factorial 5 times");
	t1=tStart();
	for (i=0; i<5; i++)
		for (j=0; j<BIG_FACT; j++)
			{
			f=factorial(j);
			/* f=j! */
			}
	t2=tGet();
	printf("Computing all factorials from 0..n "
		   "5 times took %s seconds.\n\n",
		   tFormat(t1,t2));

	/* STEP 2 - compute the table, then look up	*
	 *			each factorial 5 times.			*/
	puts("Now compute table, and look up each "
		 "factorial 5 times.");
	t1=tStart();
	for (i=0; i<BIG_FACT; i++)
		table[i]=factorial(i);
	t2=tGet();
	for (i=0; i<5; i++)
		for (j=0; j<BIG_FACT; j++)
			{
			f=table[j];
			/* f=j! */
			}
	t3=tGet();
	printf("Computing table took %s seconds\n",
			tFormat(t1,t2));
	printf("Looking up each factorial 5 times to"
			"ok %s seconds\n",tFormat(t2,t3));
	printf("Total: %s seconds\n",tFormat(t1,t3));

	/* STEP 3 - Compute each factorial as it is	*
	 *			needed.							*/
	puts("Clearing the table, and computing each "
		 "factorial as it is needed (for 5) "
		 "lookups.");
	t1=tStart();
	for (i=0; i<BIG_FACT; i++)
		table[i]=-1.0;				/* 'unknown' val*/
	for (i=0; i<5; i++)
		for (j=0; j<BIG_FACT; j++)
			{
			f=table[j];
			if (f<0.0)
				{
				f=factorial(j);
				table[j]=f;
				}
			/* f=j!*/
			}
	t2=tGet();
	printf("Clearing the table and computing each"
		   " factorial as it was needed for 5\n"
		   "lookups took %s seconds.\n\n",
		   tFormat(t1,t2));

	/* STEP 4 - write the table to disk (we are	*
	 *			not timing this step, because if*
	 *			you are loading it from disk,	*
	 *			you presumably don't care how	*
	 *			long it took to comput it.		*/
	if((of=fopen("FACT.LUT","wb")) == NULL)
		{
		puts("ERROR! could open FACT.LUT");
		exit(TRUE);
		}
	if (fwrite(table,szDBL,BIG_FACT,of)!=BIG_FACT)
		{
		puts("ERROR! could not write FACT.LUT");
		exit(TRUE);
		}
	fclose(of);
	/* Flush the disk buffer, so that the time	*
	 * is not affected by having the data in a	*
	 * disk buffer.								*/
	system("CHKDSK");

	/* STEP 5 - read the table from disk, and	*
	 *			use each factorial 5 times		*/
	t1=tStart();
	if ((of=fopen("FACT.LUT","rb"))==NULL)
		{
		puts("ERROR! couldn't open FACT.LUT");
		exit(TRUE);
		}
	if (fread(table,szDBL,BIG_FACT,of)!=BIG_FACT)
		{
		puts("ERROR! couldnot read FACT.LUT");
		exit(TRUE);
		}
	fclose(of);
	t2=tGet();
	for (i=0; i<5; i++)
		for (j=0; j<BIG_FACT; j++)
			{
			f=table[j];
			/* f=j! */
			}
	t3=tGet();
	printf("Reading the table from disk took %s "
			"seconds\n",tFormat(t1,t2));
	printf("Looking up each factorial 5 times to"
			"ok %s seconds\n",tFormat(t2,t3));
	printf("Total: %s seconds\n",tFormat(t1,t3));
	}
