/************************************************
* TIMER - Fine resolution timer functions		*
************************************************/

#include <stdio.h>
#include <dos.h>		/* _dos_setvect, getvect*/
#include <math.h>		/* fmod					*/
#include "timer.h"

#define TIX_SEC    18.20648193
#define TIX_MIN    (TIX_SEC*60.0)
#define TIX_HOUR   (TIX_MIN*60.0)
#define TIX_DAY    (TIX_HOUR*24.0)

long far *_tTickCount=MK_FP(0x0040,0x006c);

/************************************************
* tStart - wait for a new tick, and return the	*
* tick number to the caller.  The wait allows	*
* us to be sure the user gets a start at the	*
* beginning of the second.						*
************************************************/
long tStart(void)
	{
	long StartTime;

	StartTime=tGet();
	while (StartTime == tGet())
		;
	return tGet();
	}


/************************************************
* tDiff - compute the difference between two	*
* timepoints (in seconds).						*
*												*
* INP:	t1 - start time (from t1Start or tGet)	*
*		t2 - end time (from t1Get)				*
*												*
* OUT:	a double containing # of seconds between*
*		t1 and t2.								*
************************************************/
double tDiff(long StartTime, long EndTime)
	{
	return (double)(EndTime-StartTime)/TIX_SEC;
	}

/************************************************
* tFormat - given two times, return a pointer	*
* to a (static) string that is the difference	*
* in the times, formatted {{DD:}HH:}MM:SS		*
*												*
* INP:	t1 - the first time (from t1Start)		*
*		t2 - the end time (from t1Get)			*
* OUT:	pointer to the string with the formatted*
*		string.									*
************************************************/
char *tFormat(long t1, long t2)
	{
	static char TimeFmt[20];
	int		dd, hh, mm, ss, sf;
	double	temp;

	temp=(double)(t2-t1);		/* Time diff.	*/

	dd=(int)(temp/TIX_DAY);		/* days			*/
	if (dd>99) dd=99;			/* Days ovrflw?	*/
	temp=fmod(temp,TIX_DAY);
	hh=(int)(temp/TIX_HOUR);	/* hours		*/
	temp=fmod(temp,TIX_HOUR);
	mm=(int)(temp/TIX_MIN);		/* minutes		*/
	temp=fmod(temp,TIX_MIN);
	ss=(int)(temp/TIX_SEC);		/* seconds		*/
	temp=fmod(temp,TIX_SEC);	/* milliseconds	*/
	sf=(int)(temp*100.0/TIX_SEC+0.5);

	sprintf(TimeFmt,"%02d:%02d:%02d:%02d.%02d",
					  dd,  hh,  mm,  ss,  sf);
	if (dd!=0) return TimeFmt;
	if (hh!=0) return TimeFmt+3;
	return TimeFmt+6;
	}
